<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\API\Role\CreateOrUpdateRequest;
use App\Http\Resources\API\RoleResource;
use App\Models\Menu;
use App\Models\Role;
use Illuminate\Http\Request;

class RoleController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $roles = Role::all();

        return RoleResource::collection($roles);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(CreateOrUpdateRequest $request)
    {
        Role::create([
            ...$request->only(['name', 'description']),
            'guard_name' => 'api',
        ]);

        return $this->created();
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
        $role=Role::query()->findOrFail($id);

        return new RoleResource($role);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(CreateOrUpdateRequest $request, string $id)
    {
        $role = Role::query()->findOrFail($id);

        $role->update($request->only([
            'name', 'description'
        ]));

        return $this->noContent();
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        Role::destroy($id);

        return $this->noContent();
    }

    public function permissions(string $id)
    {
        $role = Role::query()->findOrFail($id);

        return $this->success([
            'data' => $role->permissions?->pluck('name'),
        ]);
    }

    public function assignPermissions(string $id, Request $request)
    {
        $role = Role::query()->findOrFail($id);

        $role->syncPermissions($request->input('permissions', []));

        return $this->noContent();
    }

    public function menus(string $id)
    {
        $role = Role::query()->findOrFail($id);

        $menus = Menu::query()->where("group", \request("group", "web"))
            ->get();

        $rolePermission=$role->permissions->pluck('name')->toArray();

        $flattenedPaths = [];
        foreach ($menus as $index=> $menu) {
            if(in_array($menu->permission,$rolePermission)){
                $individualPaths = explode(',', $menu->path);
                $flattenedPaths = array_merge($flattenedPaths, $individualPaths);
            }
        }

        $roleHasMenu=Menu::query()->whereIn('id',$flattenedPaths)->pluck('name')->toArray();

        return $roleHasMenu;

    }


    public function assignMenusPermissions(string $id, Request $request)
    {
        $role = Role::query()->findOrFail($id);

        $permissions=$request->input('permissions');
        foreach ($permissions as $permission){
            if($role->hasPermissionTo($permission)){
                $role->revokePermissionTo($permission);
            }else{
                $role->givePermissionTo($permission);
            };
        }

        return $this->noContent();
    }
}
